@@ -1,74 +1,86 @@
 /*
- * yeti_misc.c --
+ * yeti_misc.c -
  *
- *	Implement miscellaneous builtin functions in Yeti.
+ * Implement miscellaneous builtin functions in Yeti.
  *
  *-----------------------------------------------------------------------------
  *
- *	Copyright (C) 1999-2008 Eric Thibaut.
+ * Copyright (C) 1996-2010 Eric Thiébaut <thiebaut@obs.univ-lyon1.fr>
  *
- *	This file is part of Yeti.
+ * This software is governed by the CeCILL-C license under French law and
+ * abiding by the rules of distribution of free software.  You can use, modify
+ * and/or redistribute the software under the terms of the CeCILL-C license as
+ * circulated by CEA, CNRS and INRIA at the following URL
+ * "http://www.cecill.info".
  *
- *	Yeti is  free software;  you can redistribute  it and/or  modify it
- *	under  the terms of  the GNU  General Public  License version  2 as
- *	published by the Free Software Foundation.
+ * As a counterpart to the access to the source code and rights to copy,
+ * modify and redistribute granted by the license, users are provided only
+ * with a limited warranty and the software's author, the holder of the
+ * economic rights, and the successive licensors have only limited liability.
  *
- *	Yeti is distributed in the hope that it will be useful, but WITHOUT
- *	ANY WARRANTY; without even  the implied warranty of MERCHANTABILITY
- *	or FITNESS  FOR A PARTICULAR  PURPOSE.  See the GNU  General Public
- *	License for more details.
+ * In this respect, the user's attention is drawn to the risks associated with
+ * loading, using, modifying and/or developing or reproducing the software by
+ * the user in light of its specific status of free software, that may mean
+ * that it is complicated to manipulate, and that also therefore means that it
+ * is reserved for developers and experienced professionals having in-depth
+ * computer knowledge. Users are therefore encouraged to load and test the
+ * software's suitability as regards their requirements in conditions enabling
+ * the security of their systems and/or data to be ensured and, more
+ * generally, to use and operate it in the same conditions as regards
+ * security.
  *
- *	You should have  received a copy of the  GNU General Public License
- *	along with  Yeti (file "COPYING"  in the top source  directory); if
- *	not, write to  the Free Software Foundation, Inc.,  51 Franklin St,
- *	Fifth Floor, Boston, MA 02110-1301 USA
+ * The fact that you are presently reading this means that you have had
+ * knowledge of the CeCILL-C license and that you accept its terms.
  *
  *-----------------------------------------------------------------------------
  *
- * History:
- *	$Id: yeti_misc.c,v 1.10 2008/04/02 14:11:25 eric Exp $
- *	$Log: yeti_misc.c,v $
- *	Revision 1.10  2008/04/02 14:11:25  eric
- *	 - Fix symbol_names not reporting scalar symbols.
+ * $Id: yeti_misc.c,v 1.11 2010/04/13 14:41:39 eric Exp $
+ * $Log: yeti_misc.c,v $
+ * Revision 1.11  2010/04/13 14:41:39  eric
+ *  - Changed license.
+ *  - Functions that are now in Yorick have been removed.
  *
- *	Revision 1.9  2008/02/14 11:14:59  eric
- *	- Fix functions: identof, is_scalar, is_vector, is_matrix,
- *	  is_integer, is_real, is_complex, is_string, and
- *	  is_numerical when argument is an L-value. Thanks to
- *	  "sguieu" for reporting this bug on Yorick forum at
- *	  SourceForge.
- *	- Functions mem_copy and mem_copy also fixed to prevent
- *	  this problem.
+ * Revision 1.10  2008/04/02 14:11:25  eric
+ *  - Fix symbol_names not reporting scalar symbols.
  *
- *	Revision 1.8  2007/12/26 10:59:28  eric
- *	 - Update for management of dimension lists.
- *	 - Fix encoding.
- *	 - Minor fix in window_geometry.
+ * Revision 1.9  2008/02/14 11:14:59  eric
+ * - Fix functions: identof, is_scalar, is_vector, is_matrix,
+ *   is_integer, is_real, is_complex, is_string, and
+ *   is_numerical when argument is an L-value. Thanks to
+ *   "sguieu" for reporting this bug on Yorick forum at
+ *   SourceForge.
+ * - Functions mem_copy and mem_copy also fixed to prevent
+ *   this problem.
  *
- *	Revision 1.7  2007/07/27 07:43:58  eric
- *	 - Changes to globalize version number in the form:
- *	   MAJOR.MINOR.MICRO[SUFFIX] -- i.e. SUFFIX is optional.
+ * Revision 1.8  2007/12/26 10:59:28  eric
+ *  - Update for management of dimension lists.
+ *  - Fix encoding.
+ *  - Minor fix in window_geometry.
  *
- *	Revision 1.6  2007/05/01 20:23:39  eric
- *	 - Fixed some compiler warnings.
+ * Revision 1.7  2007/07/27 07:43:58  eric
+ *  - Changes to globalize version number in the form:
+ *    MAJOR.MINOR.MICRO[SUFFIX] -- i.e. SUFFIX is optional.
  *
- *	Revision 1.5  2007/04/24 07:58:07  eric
- *	 - The `symbol_names` function can now specifically select
- *	   lists, hash tables and/or auto-loaded functions.
+ * Revision 1.6  2007/05/01 20:23:39  eric
+ *  - Fixed some compiler warnings.
  *
- *	Revision 1.4  2006/12/05 07:19:51  eric
- *	 - Renamed built-in `typeIDof` as `identof`.
+ * Revision 1.5  2007/04/24 07:58:07  eric
+ *  - The `symbol_names` function can now specifically select
+ *    lists, hash tables and/or auto-loaded functions.
  *
- *	Revision 1.3  2006/07/19 17:33:11  eric
- *	 - New built-in function insure_temporary.
+ * Revision 1.4  2006/12/05 07:19:51  eric
+ *  - Renamed built-in `typeIDof` as `identof`.
  *
- *	Revision 1.2  2006/07/19 14:50:13  eric
- *	 - Copyright notice updated.
- *	 - Many new built-in functions: is_scalar, is_vector, etc.
- *	 - Changes in Yeti initialization.
+ * Revision 1.3  2006/07/19 17:33:11  eric
+ *  - New built-in function insure_temporary.
  *
- *	Revision 1.1  2005/05/24 13:24:44  eric
- *	Initial revision
+ * Revision 1.2  2006/07/19 14:50:13  eric
+ *  - Copyright notice updated.
+ *  - Many new built-in functions: is_scalar, is_vector, etc.
+ *  - Changes in Yeti initialization.
+ *
+ * Revision 1.1  2005/05/24 13:24:44  eric
+ * Initial revision
  */
 
 #include <stdlib.h>
@@ -76,11 +88,10 @@
 #include <ctype.h>
 #include <float.h>
 
+#include <play.h>
+#include <pstdio.h>
 #include <yapi.h>
 #include <yio.h>
-#include <pstdio.h>
-#include <hlevel.h>
-#include <xfancy.h>
 
 #include "config.h"
 #include "yeti.h"
@@ -99,27 +110,11 @@
 /* Built-in functions defined in this file: */
 extern BuiltIn Y__yeti_init;
 extern BuiltIn Y_mem_base, Y_mem_copy, Y_mem_peek;
-extern BuiltIn Y_window_geometry;
 extern BuiltIn Y_set_alarm;
 extern BuiltIn Y_get_encoding;
-extern BuiltIn Y_symbol_exists, Y_symbol_names;
-extern BuiltIn Y_unref, Y_swap;
-extern BuiltIn Y_get_includes, Y_current_include;
-extern BuiltIn Y_filepath;
-extern BuiltIn Y_identof, Y_nrefsof;
+extern BuiltIn Y_nrefsof;
 extern BuiltIn Y_smooth3;
-#if PROVIDE_IS_LIST
-extern BuiltIn Y_is_list;
-#endif /* PROVIDE_IS_LIST */
-extern BuiltIn Y_is_scalar, Y_is_vector, Y_is_matrix;
-extern BuiltIn Y_is_integer, Y_is_real, Y_is_complex, Y_is_string;
-extern BuiltIn Y_is_numerical;
-
-#if 0
-/* Yorick functions defined elsewhere: */
-PLUG_API void BuildDimList(Symbol *stack, int argc);  /* ops3.c */
-PLUG_API DataBlock *ForceToDB(Symbol *s);             /* ops3.c */
-#endif
+extern BuiltIn Y_insure_temporary;
 
 /*---------------------------------------------------------------------------*/
 /* INITIALIZATION OF YETI */
@@ -614,141 +609,6 @@ static Operand *form_operand_db(Symbol *owner, Operand *op)
   return op;
 }
 
-/*---------------------------------------------------------------------------*/
-/* GIST WINDOW */
-
-#ifndef GH_NDEVS
-# define GH_NDEVS 8 /* max number of graphic windows in old Yorick */
-#endif
-
-extern BuiltIn Y_window_geometry;
-void Y_window_geometry(int argc)
-{
-  int win;
-  double *geom;
-  XEngine *engine;
-  GpXYMap *map;
-
-  GpTransform *transform;
-  double one_pixel, dpi, xbias, ybias, width, height;
-
-  if (argc != 1) {
-    YError("window_geometry takes exactly one, possibly nil, argument");
-  }
-  if (YNotNil(sp)) {
-    win = (int)YGetInteger(sp);
-  } else {
-    win = GhGetPlotter();
-  }
-  if (win < 0 || win >= GH_NDEVS || ! ghDevices[win].display) {
-    PushDataBlock(RefNC(&nilDB));
-    return;
-  }
-
-  /* NDC -> pixel coordinate transform:
-   *   XPIX = (int)(XSCALE*XNDC + XOFFSET)
-   *   YPIX = (int)(YSCALE*YNDC + YOFFSET)
-   * with:
-   *   XSCALE = ENGINE->map.x.scale    XOFFSET = ENGINE->map.x.offset - margin
-   *   YSCALE = ENGINE->map.y.scale    YOFFSET = ENGINE->map.y.offset - margin
-   * assuming:
-   *   (XSCALE*XNDC + XOFFSET) >= 0
-   *   (YSCALE*YNDC + YOFFSET) >= 0
-   * the reverse transform is:
-   *   XPIX <= XSCALE*XNDC + XOFFSET < XPIX + 1
-   *   YPIX <= YSCALE*YNDC + YOFFSET < YPIX + 1
-   * to avoid rounding errors we choose the middle of the interval:
-   *   XNDC  =  (XPIX - XOFFSET + 0.5)/XSCALE  =  XBIAS + XPIX*ONE_PIXEL
-   *   YNDC  =  (YPIX - YOFFSET + 0.5)/YSCALE  =  YBIAS - YPIX*ONE_PIXEL
-   * with:
-   *   ONE_PIXEL = 1.0/XSCALE = -1.0/YSCALE
-   *       XBIAS = (0.5 - XOFFSET)*ONE_PIXEL
-   *       XBIAS = (YOFFSET - 0.5)*ONE_PIXEL
-   */
-
-  engine = (XEngine *)ghDevices[win].display;
-  if (engine) {
-    map= &engine->e.map;
-    transform = &(engine->e.transform);
-    dpi = engine->dpi;
-    one_pixel = 2.0/(map->x.scale - map->y.scale);
-#define MARGIN(SIDE) (engine->SIDE##Margin)
-    xbias = (MARGIN(left) - map->x.offset + 0.5)/map->x.scale;
-    ybias = (MARGIN(top)  - map->y.offset + 0.5)/map->y.scale;
-#undef MARGIN
-#if (YORICK_VERSION_MAJOR == 1) && (YORICK_VERSION_MINOR <= 4)
-    fprintf(stderr, "/* XEngine */ width=%d; height=%d;\nleftMargin=%d; topMargin=%d;\nx=%d; y=%d; \n",
-	    engine->width, engine->height,
-	    engine->leftMargin, engine->topMargin,
-	    engine->x, engine->y);
-#else
-    width = engine->wtop;
-    height = engine->htop;
-#endif
-  } else {
-    dpi = one_pixel = xbias = ybias = width = height = 0.0;
-  }
-
-  /* Build result array: [DPI, ONE_PIXEL, XBIAS, YBIAS, WIDTH, HEIGHT] */
-  geom = YETI_PUSH_NEW_D(yeti_start_dimlist(6));
-  geom[0] = dpi;
-  geom[1] = one_pixel;
-  geom[2] = xbias;
-  geom[3] = ybias;
-  geom[4] = width;
-  geom[5] = height;
-}
-
-extern BuiltIn Y_window_exists;
-void Y_window_exists(int argc)
-{
-  long n;
-  if (argc != 1) YError("window_exists takes exactly one argument");
-  n = YGetInteger(sp);
-  PushIntValue(((n >= 0 && n < GH_NDEVS && ghDevices[n].display) ? 1 : 0));
-}
-
-extern BuiltIn Y_window_select;
-void Y_window_select(int argc)
-{
-  int n;
-  if (argc != 1) YError("window_select takes exactly one argument");
-  n = (int)YGetInteger(sp);
-  if (n >= 0 && n < GH_NDEVS && ghDevices[n].display) {
-    GhSetPlotter(n);
-    PushIntValue(1);
-  } else {
-    PushIntValue(0);
-  }
-}
-
-extern BuiltIn Y_window_list;
-void Y_window_list(int argc)
-{
-  long *p, i, n, dims[2];
-
-  if (argc != 1 || YNotNil(sp)) {
-    YError("window_list takes exactly one nil argument");
-  }
-  for (n=i=0 ; i<GH_NDEVS ; ++i) {
-    if (ghDevices[i].display) {
-      ++n;
-    }
-  }
-  if (n >= 1) {
-    dims[0] = 1;
-    dims[1] = n;
-    p = ypush_l(dims);
-    for (n=i=0 ; i<GH_NDEVS ; ++i) {
-      if (ghDevices[i].display) {
-	p[n++] = i;
-      }
-    }
-  } else {
-    ypush_nil();
-  }
-}
-
 /*---------------------------------------------------------------------------*/
 /* ALARM CALLBACK */
 
@@ -1046,154 +906,6 @@ void Y_nrefsof(int argc)
   PushLongValue(sp->ops->FormOperand(sp, &op)->references);
 }
 
-void Y_symbol_exists(int argc)
-{
-  if (argc != 1) YError("symbol_exists takes exactly one argument");
-  PushIntValue(HashFind(&globalTable, YGetString(sp), 0L));
-}
-
-#define GET_ARRAY       1
-#define GET_STRUCT      2
-#define GET_RANGE       4
-#define GET_VOID        8
-#define GET_FUNCTION   16
-#define GET_BUILTIN    32
-#define GET_STRUCTDEF  64
-#define GET_STREAM    128
-#define GET_OPAQUE    256
-#define GET_LIST      512
-#define GET_HASH     1024
-#define GET_AUTOLOAD 2048
-
-void Y_symbol_names(int argc)
-{
-  extern Operations hashOps, listOps;
-  long i, nitems, number;
-  char **ret;
-  int match[T_OPAQUE+1];
-  int type, flags, pass;
-  int omit_array, omit_list, omit_hash, omit_autoload, omit_opaque;
-
-  if (argc != 1) YError("symbol_list takes exactly one argument");
-  flags = yeti_get_optional_integer(sp, (GET_ARRAY | GET_STRUCT | GET_RANGE |
-					 GET_FUNCTION | GET_BUILTIN |
-					 GET_STRUCTDEF | GET_STREAM |
-					 GET_OPAQUE));
-  nitems = globalTable.nItems;
-  if (nitems <= 0) {
-    /* No symbols defined. */
-    PushDataBlock(RefNC(&nilDB));
-    return;
-  }
-  if (flags == -1) {
-    /* Return names of all symbols ever defined. */
-    ret = YETI_PUSH_NEW_Q(yeti_start_dimlist(nitems));
-    for (i=0 ; i<nitems ; ++i) {
-      ret[i] = p_strcpy(globalTable.names[i]);
-    }
-    return;
-  }
-  omit_array = ((flags & GET_ARRAY) == 0);
-  omit_list = ((flags & GET_LIST) == 0);
-  omit_hash = ((flags & GET_HASH) == 0);
-  omit_autoload = ((flags & GET_AUTOLOAD) == 0);
-  omit_opaque = ((flags & GET_OPAQUE) == 0);
-  if ((flags & (GET_LIST | GET_HASH | GET_AUTOLOAD)) != 0) {
-    flags |= GET_OPAQUE;
-  }
-  for (i = 0; i <= T_OPAQUE; ++i) {
-    match[i] = 0;
-  }
-  match[T_CHAR]      = ((flags & GET_ARRAY) != 0);
-  match[T_SHORT]     = ((flags & GET_ARRAY) != 0);
-  match[T_INT]       = ((flags & GET_ARRAY) != 0);
-  match[T_LONG]      = ((flags & GET_ARRAY) != 0);
-  match[T_FLOAT]     = ((flags & GET_ARRAY) != 0);
-  match[T_DOUBLE]    = ((flags & GET_ARRAY) != 0);
-  match[T_COMPLEX]   = ((flags & GET_ARRAY) != 0);
-  match[T_STRING]    = ((flags & GET_ARRAY) != 0);
-  match[T_POINTER]   = ((flags & GET_ARRAY) != 0);
-  match[T_STRUCT]    = ((flags & GET_STRUCT) != 0);
-  match[T_RANGE]     = ((flags & GET_RANGE) != 0);
-#ifdef GET_LVALUE
-  match[T_LVALUE]    = ((flags & GET_LVALUE) != 0);
-#endif
-  match[T_VOID]      = ((flags & GET_VOID) != 0);
-  match[T_FUNCTION]  = ((flags & GET_FUNCTION) != 0);
-  match[T_BUILTIN]   = ((flags & GET_BUILTIN) != 0);
-  match[T_STRUCTDEF] = ((flags & GET_STRUCTDEF) != 0);
-  match[T_STREAM]    = ((flags & GET_STREAM) != 0);
-  match[T_OPAQUE]    = ((flags & GET_OPAQUE) != 0);
-
-  /* Counter number of matching symbols. */
-  ret = NULL; /* avoids compiler warning */
-  number = 0;
-  for (pass = 0; pass <= 1; ++pass) {
-    if (pass) {
-      if (number <= 0) {
-	/* No matching symbols found. */
-	PushDataBlock(RefNC(&nilDB));
-	return;
-      }
-      ret = YETI_PUSH_NEW_Q(yeti_start_dimlist(number));
-    }
-    for (i=0 ; i<nitems ; ++i) {
-      OpTable *sym_ops = globTab[i].ops;
-      if (sym_ops == &dataBlockSym) {
-	Operations *ops = globTab[i].value.db->ops;
-	type = ops->typeID;
-	if ((unsigned int)type > T_OPAQUE || ! match[type]) {
-	  continue;
-	}
-	if (type == T_OPAQUE) {
-	  if (ops == &listOps) {
-	    if (omit_list) {
-	      continue;
-	    }
-	  } else if (ops == &hashOps) {
-	    if (omit_hash) {
-	      continue;
-	    }
-	  } else if (ops == &auto_ops) {
-	    if (omit_autoload) {
-	      continue;
-	    }
-	  } else {
-	    if (omit_opaque) {
-	      continue;
-	    }
-	  }
-	}
-      } else if (sym_ops == &longScalar ||
-		 sym_ops == &intScalar ||
-		 sym_ops == &doubleScalar) {
-	if (omit_array) {
-	  continue;
-	}
-      }
-      if (pass) {
-	*ret++ = p_strcpy(globalTable.names[i]);
-      } else {
-	++number;
-      }
-    }
-  }
-}
-#undef GET_ARRAY
-#undef GET_STRUCT
-#undef GET_RANGE
-#undef GET_VOID
-#undef GET_FUNCTION
-#undef GET_BUILTIN
-#undef GET_STRUCTDEF
-#undef GET_STREAM
-#undef GET_OPAQUE
-#undef GET_LIST
-#undef GET_HASH
-#undef GET_AUTOLOAD
-
-extern BuiltIn Y_insure_temporary;
-
 void Y_insure_temporary(int argc)
 {
   OpTable *ops;
@@ -1339,330 +1051,6 @@ static int get_dims(Symbol *s, Dimension **dims)
 
 #endif
 
-static int get_type(Symbol *s);
-/* Returns the type identifier (typeID) of stack symbol S, taking care of
-   L-values and of following references.  Keyword symbols yield -1. */
-
-static int get_type(Symbol *s)
-{
-  for (;;) {
-    if (s->ops == &dataBlockSym) {
-      DataBlock *db = s->value.db;
-      if (db->ops == &lvalueOps) {
-	return ((LValue *)db)->type.base->dataOps->typeID;
-      } else {
-        return db->ops->typeID;
-      }
-    } else if (s->ops == &referenceSym) {
-      s = &globTab[s->index];
-    } else if (s->ops == &doubleScalar) {
-      return T_DOUBLE;
-    } else if (s->ops == &longScalar) {
-      return T_LONG;
-    } else if (s->ops == &intScalar) {
-      return T_INT;
-    } else {
-      /* Must be a keyword. */
-      return -1;
-    }
-  }
-}
-
-void Y_identof(int argc)
-{
-  long type;
-  if (argc != 1) YError("identof takes exactly one argument");
-  type = get_type(sp);
-  if (type == -1) YError("unexpected keyword argument");
-  PushLongValue(type);
-}
-
-void Y_is_scalar(int nargs)
-{
-  Operand op;
-  Symbol *s;
-  Operations *ops;
-  int result;
-
-  if (nargs != 1) YError("is_scalar takes exactly one argument");
-  s = sp;
-  for (;;) {
-    if (s->ops == &dataBlockSym) {
-      ops =  s->value.db->ops;
-      result = ((ops->isArray || ops == &lvalueOps)
-		&& s->ops->FormOperand(s, &op)->type.dims == (Dimension *)0);
-      break;
-    } else if (s->ops != &referenceSym) {
-      /* Must be one of: intScalar, longScalar, or doubleScalar. */
-      result = 1;
-      break;
-    }
-    s = &globTab[s->index];
-  }
-  PushIntValue(result);
-}
-
-void Y_is_vector(int nargs)
-{
-  Operand op;
-  Symbol *s;
-  Dimension *dims;
-  Operations *ops;
-  int result;
-
-  if (nargs != 1) YError("is_vector takes exactly one argument");
-  s = sp;
-  for (;;) {
-    if (s->ops == &dataBlockSym) {
-      ops =  s->value.db->ops;
-      result = ((ops->isArray || ops == &lvalueOps)
-		&& (dims = s->ops->FormOperand(s, &op)->type.dims) != (Dimension *)0
-		&& dims->next == (Dimension *)0);
-      break;
-    } else if (s->ops != &referenceSym) {
-      /* Must be one of: intScalar, longScalar, or doubleScalar. */
-      result = 0;
-      break;
-    }
-    s = &globTab[s->index];
-  }
-  PushIntValue(result);
-}
-
-void Y_is_matrix(int nargs)
-{
-  Operand op;
-  Symbol *s;
-  Dimension *dims;
-  Operations *ops;
-  int result;
-
-  if (nargs != 1) YError("is_matrix takes exactly one argument");
-  s = sp;
-  for (;;) {
-    if (s->ops == &dataBlockSym) {
-      ops =  s->value.db->ops;
-      result = ((ops->isArray || ops == &lvalueOps)
-		&& (dims = s->ops->FormOperand(s, &op)->type.dims) != (Dimension *)0
-		&& dims->next != (Dimension *)0 && dims->next->next == (Dimension *)0);
-      break;
-    } else if (s->ops != &referenceSym) {
-      /* Must be one of: intScalar, longScalar, or doubleScalar. */
-      result = 0;
-      break;
-    }
-    s = &globTab[s->index];
-  }
-  PushIntValue(result);
-}
-
-#if YETI_MUST_PROVIDE_IS_LIST
-void Y_is_list(int nargs)
-{
-  Symbol *s;
-  if (nargs != 1) YError("is_list takes exactly one argument");
-  s = YETI_DEREF_SYMBOL(sp);
-  if (s->ops == &dataBlockSym) {
-    Operations *ops = s->value.db->ops;
-#if 0 /* listOps is not exported for plugins in Yorick 1.6.02 */
-    extern Operations listOps;
-    PushIntValue(s->value.db->ops == &listOps);
-#else
-    PushIntValue(ops->typeID == T_OPAQUE && ! strcmp(ops->typeName, "list"));
-#endif
-  } else {
-    PushIntValue(0);
-  }
-}
-#endif /* YETI_MUST_PROVIDE_IS_LIST */
-
-void Y_is_integer(int nargs)
-{
-  int type;
-  if (nargs != 1) YError("is_integer takes exactly one argument");
-  type = get_type(sp);
-  PushIntValue(type >= T_CHAR && type <= T_LONG);
-}
-
-void Y_is_real(int nargs)
-{
-  int type;
-  if (nargs != 1) YError("is_real takes exactly one argument");
-  type = get_type(sp);
-  PushIntValue(type == T_DOUBLE || type == T_FLOAT);
-}
-
-void Y_is_complex(int nargs)
-{
-  if (nargs != 1) YError("is_complex takes exactly one argument");
-  PushIntValue(get_type(sp) == T_COMPLEX);
-}
-
-void Y_is_string(int nargs)
-{
-  if (nargs != 1) YError("is_string takes exactly one argument");
-  PushIntValue(get_type(sp) == T_STRING);
-}
-
-void Y_is_pointer(int nargs)
-{
-  if (nargs != 1) YError("is_pointer takes exactly one argument");
-  PushIntValue(get_type(sp) == T_POINTER);
-}
-
-void Y_is_numerical(int nargs)
-{
-  int result;
-  if (nargs != 1) YError("is_numerical takes exactly one argument");
-  switch (get_type(sp)) {
-  case T_CHAR:
-  case T_SHORT:
-  case T_INT:
-  case T_LONG:
-  case T_FLOAT:
-  case T_DOUBLE:
-  case T_COMPLEX:
-    result = 1;
-    break;
-  default:
-    result = 0;
-  }
-  PushIntValue(result);
-}
-
-/*---------------------------------------------------------------------------*/
-/* VARIABLES */
-
-void Y_unref(int argc)
-{
-  if (argc != 1) YError("unref takes exactly one argument");
-  if (sp->ops == &referenceSym) {
-    /* Replace reference without augmenting the number of references of
-       the data block object if it is an array. */
-    Symbol *ref = &globTab[sp->index];
-    OpTable *ops = ref->ops;
-    if (ops == &dataBlockSym) {
-      DataBlock *db = ref->value.db;
-      if (db && db->ops->isArray) {
-	/* Replace symbol in global table by nil. */
-	ref->value.db = RefNC(&nilDB);
-	sp->value.db = db; /* no Ref */
-      } else {
-	sp->value.db = Ref(db);
-      }
-    } else {
-      sp->value = ref->value;
-    }
-    sp->ops = ops; /* change ops only AFTER value updated */
-  } /* NOT_NEEDED: else if (!sp->ops) YError("unexpected keyword argument"); */
-}
-
-void Y_swap(int argc)
-{
-  SymbolValue a_val, b_val;
-  OpTable *a_ops, *b_ops;
-  volatile Symbol *a_sym, *b_sym;
-  if (argc!=2) YError("swap takes exactly 2 arguments");
-  a_sym = sp;
-  b_sym = sp-1;
-  if (a_sym->ops != &referenceSym || b_sym->ops != &referenceSym)
-    YError("arguments must be simple variable references");
-  a_sym = &globTab[a_sym->index];
-  a_ops = a_sym->ops;
-  a_val = a_sym->value;
-  a_sym->ops = &intScalar;
-  b_sym = &globTab[b_sym->index];
-  b_ops = b_sym->ops;
-  b_val = b_sym->value;
-  b_sym->ops = &intScalar;
-  b_sym->value = a_val;
-  a_sym->value = b_val;
-  Drop(2);
-  b_sym->ops = a_ops;
-  a_sym->ops = b_ops;
-}
-
-/*---------------------------------------------------------------------------*/
-/* PATHS */
-
-void Y_current_include(int argc)
-{
-  if (argc != 1 || YNotNil(sp))
-    YError("current_include takes exactly one nil argument");
-  if (nYpIncludes > 0 && ypIncludes[nYpIncludes-1].filename) {
-    yeti_push_string_value(ypIncludes[nYpIncludes-1].filename);
-  } else {
-    PushDataBlock(RefNC(&nilDB));
-  }
-}
-
-void Y_get_includes(int argc)
-{
-  if (argc != 1 || YNotNil(sp))
-    YError("get_includes takes exactly one nil argument");
-  if (sourceTab.nItems > 0) {
-    int i;
-    char **s = YETI_PUSH_NEW_Q(yeti_start_dimlist(sourceTab.nItems));
-    for (i=0 ; i<sourceTab.nItems ; ++i) s[i] = p_strcpy(sourceTab.names[i]);
-  } else {
-    PushDataBlock(RefNC(&nilDB));
-  }
-}
-
-/* The following definitions _must_ match those in "ascio.c" */
-PLUG_API Operations textOps;
-
-struct TextStream {
-  int references;      /* reference counter */
-  Operations *ops;     /* virtual function table */
-#if (YORICK_VERSION_MAJOR == 1) && (YORICK_VERSION_MINOR <= 4)
-  FILE *stream;        /* 0 indicates file has been closed */
-#else
-  p_file *stream;      /* 0 indicates file has been closed */
-#endif
-  char *fullname;      /* filename after YExpandName */
-  int permissions;     /* +1 read permission, +2 write permission
-			  +4 append mode, +8 binary mode, +16 pipe */
-  /* ------ begin specific text stream part ------- */
-  long lastLineRead;   /* 1-origin line number of last line read */
-  long readPosition;   /* file position (ftell) after lastLineRead */
-  long lastPosition;   /* file position (ftell) before lastLineRead --
-			  after backup, lastPosition==readPosition,
-			  and lastPosition is not valid */
-  int readWrite;       /* 0 initially, 1 after read, 2 after write */
-  long fileID;         /* unique number used to recognize this file */
-};
-
-void Y_filepath(int argc)
-{
-  Dimension *dims;
-  Operand op;
-  char **input, **output;
-  long i, n;
-
-  if (argc != 1) YError("filepath function takes exactly one argument");
-  op.ops= 0;
-  if (sp->ops) sp->ops->FormOperand(sp, &op);
-  if (op.ops == &stringOps) {
-    input = YGet_Q(sp, 0, &dims);
-    n = TotalNumber(dims);
-    output = YETI_PUSH_NEW_Q(dims);
-    for (i=0 ; i<n ; ++i) {
-      output[i] = (input[i] ? YExpandName(input[i]) : 0);
-    }
-  } else if (op.ops == &streamOps) {
-    output = YETI_PUSH_NEW_Q(NULL);
-    output[0] = p_strcpy(((IOStream *)op.value)->fullname);
-  } else if (op.ops == &textOps) {
-    output = YETI_PUSH_NEW_Q(NULL);
-    output[0] = p_strcpy(((TextStream *)op.value)->fullname);
-  } else if (op.ops == &voidOps) {
-    PushDataBlock(RefNC(&nilDB));
-  } else {
-    YError("bad argument: expecting text/binary file or file name(s)");
-  }
-}
-
 /*---------------------------------------------------------------------------*/
 /* SMOOTHING */
 
@@ -1822,11 +1210,13 @@ static void smooth_single(double *x, double p25, double p50, double p75,
   }
 }
 
-/*---------------------------------------------------------------------------*
- * Local Variables:                                                          *
- * mode: C                                                                   *
- * tab-width: 8                                                              *
- * fill-column: 75                                                           *
- * coding: latin-1                                                           *
- * End:                                                                      *
- *---------------------------------------------------------------------------*/
+/*
+ * Local Variables:
+ * mode: C
+ * tab-width: 8
+ * c-basic-offset: 2
+ * indent-tabs-mode: nil
+ * fill-column: 78
+ * coding: utf-8
+ * End:
+ */
